/**
 * 
 * pc 上传svga
 */
import React from 'react';
import styles from './index.less';
import { PageHeader, PageFooter, Phone, ShowVideoItem } from '../components';
import { blobToDataURL, mobilePreview } from '@/utils/base';
import { phoneArray, phoneItemtype, uploadToQiniu } from './common';
import iconSwitch from '@/assets/icon_switch.png';
import iconLayers from '@/assets/icon_layers.png';
import iconCode from '@/assets/icon_code.png';
import iconUpload from '@/assets/icon_upload.png';
import QRCode from 'qrcode.react';

import {
  getMediaUploadToken,
  mediaCreate,
} from '@/services/api';

class Home extends React.Component {
  state = {
    currentPhone: phoneArray[4],
    bgImage: '',
    showVideoItem: false,
    videoItem: null,
    svgaFilePath: '',
    imageFilePath: '',
    showCode: false,
    sizeToggle: false 
  };
  fileRef: React.RefObject<HTMLInputElement> = React.createRef();
  phoneRef: React.RefObject<any> = React.createRef();
  constructor(props: any) {
    super(props);
  }

  wrapClick = () => {
    this.setState({
      showCode: false,
      sizeToggle: false
    })
  }
  onSizeToggle = (e: any) => {
    e.stopPropagation();
    this.setState({
      sizeToggle: !this.state.sizeToggle
    })
  }
  onBgUploadClick = () => {
    this.fileRef.current?.click();
  };
  onFileChange = (e: any) => {
    e.stopPropagation();
    e.preventDefault();
    if (!this.fileRef || !this.fileRef.current || !this.fileRef.current.files) {
      return;
    }
    let files = this.fileRef.current.files;
    let blob = files[0];

    blobToDataURL(blob, (data: any) => {
      this.setState({
        bgImage: data,
      });
    });
  };

  // 浏览素材
  onPreviewInfo = (e: any) => {
    e.stopPropagation()
    const { videoItem } = this.phoneRef.current;
    if (!videoItem) {
      return;
    }
    this.setState({ showVideoItem: true, videoItem });
  };
  previewBack = (e: any) => {
    e.stopPropagation()
    this.setState({ showVideoItem: false });
  };

  // 上传文件生成二维码
  showQrCode = async (e: any) => {
    e.stopPropagation()
    // svga 文件
    const { videoItem } = this.phoneRef.current;
    if (!videoItem) {
      return;
    }
    this.setState({
      qrcodeLoading: true
    })
    // svga文件地址
    let svgaFilePath;
    // 背景图片地址
    let imageFilePath;
    const svgaFile = videoItem.nativeFile;
    const { data, code } = await getMediaUploadToken({ fileExtension: 'svga' });
    if (code === 'OK') {
      const res: any = await uploadToQiniu({
        key: data.key,
        token: data.upToken,
        file: svgaFile,
      });
      svgaFilePath = res.url;
      await mediaCreate({
        fileLink: res.url,
        fileType: 1,
        extension: 'svga',
      });
    }

    // 背景图片
    if (this.fileRef.current && 
      this.fileRef.current.files && 
      this.fileRef.current.files.length
    ) {
      let files = this.fileRef.current.files;
    
      let imageFile = files[0];
      const imageTokenRes = await getMediaUploadToken({ fileExtension: 'png' });
      if (imageTokenRes.code === 'OK') {
        const imgeRes: any = await uploadToQiniu({
          key: imageTokenRes.data.key,
          token: imageTokenRes.data.upToken,
          file: imageFile,
        });
        imageFilePath = imgeRes.url
        await mediaCreate({
          fileLink: imgeRes.url,
          fileType: 2,
          extendsion: 'png',
        });
      }
    }

    this.setState({
      svgaFilePath,
      imageFilePath,
      showCode: true
    })
  };

  // 切换尺寸
  onSizeChange = (item: phoneItemtype) => (e: any) => {
    e.stopPropagation()
    this.setState({
      currentPhone: item,
    });
  };

  render() {
    const { 
      currentPhone, 
      bgImage, 
      showVideoItem, 
      videoItem,
      svgaFilePath,
      imageFilePath,
      showCode,
      sizeToggle
    } = this.state;
    let _qrcodeValue = mobilePreview + '?'
    if (svgaFilePath) {
      _qrcodeValue += 'svga=' + svgaFilePath
    }
    if (imageFilePath) {
      _qrcodeValue += _qrcodeValue .indexOf('=') === -1 
      ? 'bg=' + imageFilePath
      : '&bg=' + imageFilePath
    }
    return (
      <div onClick={this.wrapClick}>
        <PageHeader />
        <div className={styles['content-wrap']} hidden={showVideoItem}>
          <div className={styles['phone-wrap']}>
            <Phone
              currentPhone={currentPhone}
              bgImage={bgImage}
              ref={this.phoneRef}
            />
          </div>

          <div className={styles['btns-wrap']}>
            <a className={styles['phone-size']} onClick={this.onSizeToggle}>
              <h3>
                <span>{currentPhone.name || '手机型号'}</span>
                <img src={iconSwitch} alt="" />
              </h3>
              <div className={styles['phone-size-wrap']} hidden={!sizeToggle}>
                <ul className={styles['phone-size-list']}>
                  {phoneArray.map((item: phoneItemtype, i: number) => (
                    <li
                      className={
                        item.name === currentPhone.name ? styles['active'] : ''
                      }
                      onClick={this.onSizeChange(item)}
                      key={i}
                    >
                      {item.name}
                    </li>
                  ))}
                </ul>
              </div>
            </a>
            <a onClick={this.onBgUploadClick}>
              <span>上传背景</span>
              <input
                type="file"
                hidden
                ref={this.fileRef}
                onChange={this.onFileChange}
              />
              <img src={iconLayers} alt="" />
            </a>
            <a onClick={this.showQrCode}>
              <span>扫码预览</span>
              <img src={iconCode} alt="" />
              {
                showCode &&
                <div className={styles['qrcode-wrap']}>
                  <QRCode 
                    value={_qrcodeValue} 
                    id='qcode'
                    size={96}
                    renderAs='canvas'
                    level='Q'
                  />
                </div>
              }
              
            </a>
            <a onClick={this.onPreviewInfo}>
              <span>浏览素材</span>
              <img src={iconUpload} alt="" />
            </a>
          </div>
        </div>

        <div className={styles['preview-wrap']} hidden={!showVideoItem}>
          <ShowVideoItem videoItem={videoItem} previewBack={this.previewBack} />
        </div>
        <PageFooter />
      </div>
    );
  }
}
export default Home;
